@@ -2397,6 +2397,7 @@ test_remove_directory (DBusString *dir)
       if (!test_remove_service_file (dir, _dbus_string_get_const_data (&filename)))
         {
           ret_val = FALSE;
+          _dbus_directory_close (iter);
           goto out;
         }
     }
@@ -2607,7 +2608,7 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
     return FALSE;
 
   if (!_dbus_string_append (&directory, "/dbus-reload-test-") ||
-      !_dbus_generate_random_ascii (&directory, 6))
+      !_dbus_generate_random_ascii (&directory, 6, NULL))
      {
        return FALSE;
      }
