@@ -24,6 +24,7 @@
 
 #include <config.h>
 #include "activation.h"
+#include "apparmor.h"
 #include "connection.h"
 #include "driver.h"
 #include "dispatch.h"
@@ -34,12 +35,21 @@
 #include "utils.h"
 
 #include <dbus/dbus-asv-util.h>
+#include <dbus/dbus-connection-internal.h>
 #include <dbus/dbus-string.h>
 #include <dbus/dbus-internals.h>
 #include <dbus/dbus-message.h>
 #include <dbus/dbus-marshal-recursive.h>
+#include <dbus/dbus-marshal-validate.h>
 #include <string.h>
 
+static inline const char *
+nonnull (const char *maybe_null,
+         const char *if_null)
+{
+  return (maybe_null ? maybe_null : if_null);
+}
+
 static DBusConnection *
 bus_driver_get_owner_of_name (DBusConnection *connection,
                               const char     *name)
@@ -118,7 +128,10 @@ bus_driver_check_caller_is_privileged (DBusConnection *connection,
 
       bus_context_log_and_set_error (bus_transaction_get_context (transaction),
           DBUS_SYSTEM_LOG_SECURITY, error, DBUS_ERROR_ACCESS_DENIED,
-          "rejected attempt to call %s by unknown uid", method);
+          "rejected attempt to call %s by connection %s (%s) with "
+          "unknown uid", method,
+          nonnull (bus_connection_get_name (connection), "(inactive)"),
+          bus_connection_get_loginfo (connection));
       return FALSE;
     }
 
@@ -139,7 +152,10 @@ bus_driver_check_caller_is_privileged (DBusConnection *connection,
 
       bus_context_log_and_set_error (bus_transaction_get_context (transaction),
           DBUS_SYSTEM_LOG_SECURITY, error, DBUS_ERROR_ACCESS_DENIED,
-          "rejected attempt to call %s by uid %lu", method, uid);
+          "rejected attempt to call %s by connection %s (%s) with "
+          "uid %lu", method,
+          nonnull (bus_connection_get_name (connection), "(inactive)"),
+          bus_connection_get_loginfo (connection), uid);
       return FALSE;
     }
 
@@ -1108,9 +1124,10 @@ bus_driver_handle_add_match (DBusConnection *connection,
                              DBusError      *error)
 {
   BusMatchRule *rule;
-  const char *text;
+  const char *text, *bustype;
   DBusString str;
   BusMatchmaker *matchmaker;
+  BusContext *context;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -1143,6 +1160,12 @@ bus_driver_handle_add_match (DBusConnection *connection,
   if (rule == NULL)
     goto failed;
 
+  context = bus_transaction_get_context (transaction);
+  bustype = context ? bus_context_get_type (context) : NULL;
+  if (bus_match_rule_get_client_is_eavesdropping (rule) &&
+      !bus_apparmor_allows_eavesdropping (connection, bustype, error))
+    goto failed;
+
   matchmaker = bus_connection_get_matchmaker (connection);
 
   if (!bus_matchmaker_add_rule (matchmaker, rule))
@@ -1646,6 +1669,7 @@ bus_driver_handle_get_connection_credentials (DBusConnection *connection,
   DBusMessageIter reply_iter;
   DBusMessageIter array_iter;
   unsigned long ulong_val;
+  char *s;
   const char *service;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
@@ -1680,6 +1704,45 @@ bus_driver_handle_get_connection_credentials (DBusConnection *connection,
         goto oom;
     }
 
+  if (dbus_connection_get_windows_user (conn, &s))
+    {
+      DBusString str;
+      dbus_bool_t result;
+
+      if (s == NULL)
+        goto oom;
+
+      _dbus_string_init_const (&str, s);
+      result = _dbus_validate_utf8 (&str, 0, _dbus_string_get_length (&str));
+      _dbus_string_free (&str);
+      if (result)
+        {
+          if (!_dbus_asv_add_string (&array_iter, "WindowsSID", s))
+            {
+              dbus_free (s);
+              goto oom;
+            }
+        }
+      dbus_free (s);
+    }
+
+  if (_dbus_connection_get_linux_security_label (conn, &s))
+    {
+      if (s == NULL)
+        goto oom;
+
+      /* use the GVariant bytestring convention for strings of unknown
+       * encoding: include the \0 in the payload, for zero-copy reading */
+      if (!_dbus_asv_add_byte_array (&array_iter, "LinuxSecurityLabel",
+                                     s, strlen (s) + 1))
+        {
+          dbus_free (s);
+          goto oom;
+        }
+
+      dbus_free (s);
+    }
+
   if (!_dbus_asv_close (&reply_iter, &array_iter))
     goto oom;
 
@@ -1692,6 +1755,8 @@ bus_driver_handle_get_connection_credentials (DBusConnection *connection,
       goto oom;
     }
 
+  dbus_message_unref (reply);
+
   return TRUE;
 
  oom:
@@ -1746,6 +1811,72 @@ bus_driver_handle_reload_config (DBusConnection *connection,
   return FALSE;
 }
 
+#ifdef DBUS_ENABLE_VERBOSE_MODE
+static dbus_bool_t
+bus_driver_handle_enable_verbose (DBusConnection *connection,
+                                  BusTransaction *transaction,
+                                  DBusMessage    *message,
+                                  DBusError      *error)
+{
+    DBusMessage *reply = NULL;
+
+    _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+    reply = dbus_message_new_method_return (message);
+    if (reply == NULL)
+      goto oom;
+
+    if (! bus_transaction_send_from_driver (transaction, connection, reply))
+      goto oom;
+
+    _dbus_set_verbose(TRUE);
+
+    dbus_message_unref (reply);
+    return TRUE;
+
+   oom:
+    _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+    BUS_SET_OOM (error);
+
+    if (reply)
+      dbus_message_unref (reply);
+    return FALSE;
+}
+
+static dbus_bool_t
+bus_driver_handle_disable_verbose (DBusConnection *connection,
+                                   BusTransaction *transaction,
+                                   DBusMessage    *message,
+                                   DBusError      *error)
+{
+    DBusMessage *reply = NULL;
+
+    _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+    reply = dbus_message_new_method_return (message);
+    if (reply == NULL)
+      goto oom;
+
+    if (! bus_transaction_send_from_driver (transaction, connection, reply))
+      goto oom;
+
+    _dbus_set_verbose(FALSE);
+
+    dbus_message_unref (reply);
+    return TRUE;
+
+   oom:
+    _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+    BUS_SET_OOM (error);
+
+    if (reply)
+      dbus_message_unref (reply);
+    return FALSE;
+}
+#endif
+
 static dbus_bool_t
 bus_driver_handle_get_id (DBusConnection *connection,
                           BusTransaction *transaction,
@@ -1808,6 +1939,8 @@ bus_driver_handle_become_monitor (DBusConnection *connection,
                                   DBusError      *error)
 {
   char **match_rules = NULL;
+  const char *bustype;
+  BusContext *context;
   BusMatchRule *rule;
   DBusList *rules = NULL;
   DBusList *iter;
@@ -1822,6 +1955,11 @@ bus_driver_handle_become_monitor (DBusConnection *connection,
   if (!bus_driver_check_message_is_for_us (message, error))
     goto out;
 
+  context = bus_transaction_get_context (transaction);
+  bustype = context ? bus_context_get_type (context) : NULL;
+  if (!bus_apparmor_allows_eavesdropping (connection, bustype, error))
+    goto out;
+
   if (!bus_driver_check_caller_is_privileged (connection, transaction,
                                               message, error))
     goto out;
@@ -2018,6 +2156,14 @@ static const MessageHandler monitoring_message_handlers[] = {
   { NULL, NULL, NULL, NULL }
 };
 
+#ifdef DBUS_ENABLE_VERBOSE_MODE
+static const MessageHandler verbose_message_handlers[] = {
+  { "EnableVerbose", "", "", bus_driver_handle_enable_verbose},
+  { "DisableVerbose", "", "", bus_driver_handle_disable_verbose},
+  { NULL, NULL, NULL, NULL }
+};
+#endif
+
 #ifdef DBUS_ENABLE_STATS
 static const MessageHandler stats_message_handlers[] = {
   { "GetStats", "", "a{sv}", bus_stats_handle_get_stats },
@@ -2050,6 +2196,9 @@ static InterfaceHandler interface_handlers[] = {
     "    </signal>\n" },
   { DBUS_INTERFACE_INTROSPECTABLE, introspectable_message_handlers, NULL },
   { DBUS_INTERFACE_MONITORING, monitoring_message_handlers, NULL },
+#ifdef DBUS_ENABLE_VERBOSE_MODE
+  { DBUS_INTERFACE_VERBOSE, verbose_message_handlers, NULL },
+#endif
 #ifdef DBUS_ENABLE_STATS
   { BUS_INTERFACE_STATS, stats_message_handlers, NULL },
 #endif
