@@ -52,7 +52,6 @@
 #include "dbus-credentials.h"
 
 #include <windows.h>
-#include <ws2tcpip.h>
 #include <wincrypt.h>
 #include <iphlpapi.h>
 
@@ -376,7 +375,7 @@ _dbus_win_free_error_string (char *string)
  */
 
 int
-_dbus_read_socket (int               fd,
+_dbus_read_socket (DBusSocket        fd,
                    DBusString       *buffer,
                    int               count)
 {
@@ -398,8 +397,8 @@ _dbus_read_socket (int               fd,
 
  again:
  
-  _dbus_verbose ("recv: count=%d fd=%d\n", count, fd);
-  bytes_read = recv (fd, data, count, 0);
+  _dbus_verbose ("recv: count=%d fd=%Iu\n", count, fd.sock);
+  bytes_read = recv (fd.sock, data, count, 0);
   
   if (bytes_read == SOCKET_ERROR)
 	{
@@ -446,7 +445,7 @@ _dbus_read_socket (int               fd,
  * @returns the number of bytes written or -1 on error
  */
 int
-_dbus_write_socket (int               fd,
+_dbus_write_socket (DBusSocket        fd,
                     const DBusString *buffer,
                     int               start,
                     int               len)
@@ -458,8 +457,8 @@ _dbus_write_socket (int               fd,
 
  again:
 
-  _dbus_verbose ("send: len=%d fd=%d\n", len, fd);
-  bytes_written = send (fd, data, len, 0);
+  _dbus_verbose ("send: len=%d fd=%Iu\n", len, fd.sock);
+  bytes_written = send (fd.sock, data, len, 0);
 
   if (bytes_written == SOCKET_ERROR)
     {
@@ -490,13 +489,13 @@ _dbus_write_socket (int               fd,
  * @returns #FALSE if error set
  */
 dbus_bool_t
-_dbus_close_socket (int        fd,
+_dbus_close_socket (DBusSocket fd,
                     DBusError *error)
 {
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
  again:
-  if (closesocket (fd) == SOCKET_ERROR)
+  if (closesocket (fd.sock) == SOCKET_ERROR)
     {
       DBUS_SOCKET_SET_ERRNO ();
       
@@ -504,11 +503,11 @@ _dbus_close_socket (int        fd,
         goto again;
         
       dbus_set_error (error, _dbus_error_from_errno (errno),
-                      "Could not close socket: socket=%d, , %s",
-                      fd, _dbus_strerror_from_errno ());
+                      "Could not close socket: socket=%Iu, , %s",
+                      fd.sock, _dbus_strerror_from_errno ());
       return FALSE;
     }
-  _dbus_verbose ("_dbus_close_socket: socket=%d, \n", fd);
+  _dbus_verbose ("_dbus_close_socket: socket=%Iu, \n", fd.sock);
 
   return TRUE;
 }
@@ -539,19 +538,19 @@ _dbus_win_handle_set_close_on_exec (HANDLE handle)
  * @returns #TRUE on success.
  */
 dbus_bool_t
-_dbus_set_fd_nonblocking (int             handle,
-                          DBusError      *error)
+_dbus_set_socket_nonblocking (DBusSocket      handle,
+                              DBusError      *error)
 {
   u_long one = 1;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
-  if (ioctlsocket (handle, FIONBIO, &one) == SOCKET_ERROR)
+  if (ioctlsocket (handle.sock, FIONBIO, &one) == SOCKET_ERROR)
     {
       DBUS_SOCKET_SET_ERRNO ();
       dbus_set_error (error, _dbus_error_from_errno (errno),
-                      "Failed to set socket %d:%d to nonblocking: %s", handle,
-                      _dbus_strerror_from_errno ());
+                      "Failed to set socket %Iu to nonblocking: %s",
+                      handle.sock, _dbus_strerror_from_errno ());
       return FALSE;
     }
 
@@ -580,7 +579,7 @@ _dbus_set_fd_nonblocking (int             handle,
  * @returns total bytes written from both buffers, or -1 on error
  */
 int
-_dbus_write_socket_two (int               fd,
+_dbus_write_socket_two (DBusSocket        fd,
                         const DBusString *buffer1,
                         int               start1,
                         int               len1,
@@ -619,8 +618,8 @@ _dbus_write_socket_two (int               fd,
 
  again:
  
-  _dbus_verbose ("WSASend: len1+2=%d+%d fd=%d\n", len1, len2, fd);
-  rc = WSASend (fd, 
+  _dbus_verbose ("WSASend: len1+2=%d+%d fd=%Iu\n", len1, len2, fd.sock);
+  rc = WSASend (fd.sock,
                 vectors,
                 data2 ? 2 : 1, 
                 &bytes_written,
@@ -643,12 +642,6 @@ _dbus_write_socket_two (int               fd,
   return bytes_written;
 }
 
-dbus_bool_t
-_dbus_socket_is_invalid (int fd)
-{
-    return fd == INVALID_SOCKET ? TRUE : FALSE;
-}
-
 #if 0
 
 /**
@@ -1045,20 +1038,22 @@ failed:
  ************************************************************************/
 
 /**
- * Creates a full-duplex pipe (as in socketpair()).
- * Sets both ends of the pipe nonblocking.
+ * Creates pair of connect sockets (as in socketpair()).
+ * Sets both ends of the pair nonblocking.
+ *
+ * Marks both file descriptors as close-on-exec
  *
  * @param fd1 return location for one end
  * @param fd2 return location for the other end
- * @param blocking #TRUE if pipe should be blocking
+ * @param blocking #TRUE if pair should be blocking
  * @param error error return
  * @returns #FALSE on failure (if error is set)
  */
 dbus_bool_t
-_dbus_full_duplex_pipe (int        *fd1,
-                        int        *fd2,
-                        dbus_bool_t blocking,
-                        DBusError  *error)
+_dbus_socketpair (DBusSocket *fd1,
+                  DBusSocket *fd2,
+                  dbus_bool_t blocking,
+                  DBusError  *error)
 {
   SOCKET temp, socket1 = -1, socket2 = -1;
   struct sockaddr_in saddr;
@@ -1139,11 +1134,11 @@ _dbus_full_duplex_pipe (int        *fd1,
         }
     }
 
-  *fd1 = socket1;
-  *fd2 = socket2;
+  fd1->sock = socket1;
+  fd2->sock = socket2;
 
-  _dbus_verbose ("full-duplex pipe %d:%d <-> %d:%d\n",
-                 *fd1, socket1, *fd2, socket2);
+  _dbus_verbose ("full-duplex pipe %Iu:%Iu <-> %Iu:%Iu\n",
+                 fd1->sock, socket1, fd2->sock, socket2);
 
   closesocket (temp);
 
@@ -1207,12 +1202,12 @@ _dbus_poll (DBusPollFD *fds,
 
 
       if (fdp->events & _DBUS_POLLIN)
-        msgp += sprintf (msgp, "R:%d ", fdp->fd);
+        msgp += sprintf (msgp, "R:%Iu ", fdp->fd.sock);
 
       if (fdp->events & _DBUS_POLLOUT)
-        msgp += sprintf (msgp, "W:%d ", fdp->fd);
+        msgp += sprintf (msgp, "W:%Iu ", fdp->fd.sock);
 
-      msgp += sprintf (msgp, "E:%d\n\t", fdp->fd);
+      msgp += sprintf (msgp, "E:%Iu\n\t", fdp->fd.sock);
 
       // FIXME: more robust code for long  msg
       //        create on heap when msg[] becomes too small
@@ -1239,7 +1234,7 @@ _dbus_poll (DBusPollFD *fds,
       if (fdp->events & _DBUS_POLLOUT)
         lNetworkEvents |= FD_WRITE | FD_CONNECT;
 
-      WSAEventSelect(fdp->fd, ev, lNetworkEvents);
+      WSAEventSelect(fdp->fd.sock, ev, lNetworkEvents);
 
       pEvents[i] = ev;
     }
@@ -1271,7 +1266,7 @@ _dbus_poll (DBusPollFD *fds,
 
           fdp->revents = 0;
 
-          WSAEnumNetworkEvents(fdp->fd, pEvents[i], &ne);
+          WSAEnumNetworkEvents(fdp->fd.sock, pEvents[i], &ne);
 
           if (ne.lNetworkEvents & (FD_READ | FD_ACCEPT | FD_CLOSE))
             fdp->revents |= _DBUS_POLLIN;
@@ -1283,20 +1278,20 @@ _dbus_poll (DBusPollFD *fds,
             fdp->revents |= _DBUS_POLLERR;
 
           if (ne.lNetworkEvents & (FD_READ | FD_ACCEPT | FD_CLOSE))
-              msgp += sprintf (msgp, "R:%d ", fdp->fd);
+              msgp += sprintf (msgp, "R:%Iu ", fdp->fd.sock);
 
           if (ne.lNetworkEvents & (FD_WRITE | FD_CONNECT))
-              msgp += sprintf (msgp, "W:%d ", fdp->fd);
+              msgp += sprintf (msgp, "W:%Iu ", fdp->fd.sock);
 
           if (ne.lNetworkEvents & (FD_OOB))
-              msgp += sprintf (msgp, "E:%d ", fdp->fd);
+              msgp += sprintf (msgp, "E:%Iu ", fdp->fd.sock);
 
           msgp += sprintf (msgp, "lNetworkEvents:%d ", ne.lNetworkEvents);
 
           if(ne.lNetworkEvents)
             ret++;
 
-          WSAEventSelect(fdp->fd, pEvents[i], 0);
+          WSAEventSelect(fdp->fd.sock, pEvents[i], 0);
         }
 
       msgp += sprintf (msgp, "\n");
@@ -1325,7 +1320,7 @@ _dbus_poll (DBusPollFD *fds,
   char *msgp;
 
   fd_set read_set, write_set, err_set;
-  int max_fd = 0;
+  SOCKET max_fd = 0;
   int i;
   struct timeval tv;
   int ready;
@@ -1344,12 +1339,12 @@ _dbus_poll (DBusPollFD *fds,
 
 
       if (fdp->events & _DBUS_POLLIN)
-        msgp += sprintf (msgp, "R:%d ", fdp->fd);
+        msgp += sprintf (msgp, "R:%Iu ", fdp->fd.sock);
 
       if (fdp->events & _DBUS_POLLOUT)
-        msgp += sprintf (msgp, "W:%d ", fdp->fd);
+        msgp += sprintf (msgp, "W:%Iu ", fdp->fd.sock);
 
-      msgp += sprintf (msgp, "E:%d\n\t", fdp->fd);
+      msgp += sprintf (msgp, "E:%Iu\n\t", fdp->fd.sock);
 
       // FIXME: more robust code for long  msg
       //        create on heap when msg[] becomes too small
@@ -1367,14 +1362,14 @@ _dbus_poll (DBusPollFD *fds,
       DBusPollFD *fdp = &fds[i]; 
 
       if (fdp->events & _DBUS_POLLIN)
-        FD_SET (fdp->fd, &read_set);
+        FD_SET (fdp->fd.sock, &read_set);
 
       if (fdp->events & _DBUS_POLLOUT)
-        FD_SET (fdp->fd, &write_set);
+        FD_SET (fdp->fd.sock, &write_set);
 
-      FD_SET (fdp->fd, &err_set);
+      FD_SET (fdp->fd.sock, &err_set);
 
-      max_fd = MAX (max_fd, fdp->fd);
+      max_fd = MAX (max_fd, fdp->fd.sock);
     }
 
   // Avoid random lockups with send(), for lack of a better solution so far
@@ -1402,14 +1397,14 @@ _dbus_poll (DBusPollFD *fds,
           {
             DBusPollFD *fdp = &fds[i];
 
-            if (FD_ISSET (fdp->fd, &read_set))
-              msgp += sprintf (msgp, "R:%d ", fdp->fd);
+            if (FD_ISSET (fdp->fd.sock, &read_set))
+              msgp += sprintf (msgp, "R:%Iu ", fdp->fd.sock);
 
-            if (FD_ISSET (fdp->fd, &write_set))
-              msgp += sprintf (msgp, "W:%d ", fdp->fd);
+            if (FD_ISSET (fdp->fd.sock, &write_set))
+              msgp += sprintf (msgp, "W:%Iu ", fdp->fd.sock);
 
-            if (FD_ISSET (fdp->fd, &err_set))
-              msgp += sprintf (msgp, "E:%d\n\t", fdp->fd);
+            if (FD_ISSET (fdp->fd.sock, &err_set))
+              msgp += sprintf (msgp, "E:%Iu\n\t", fdp->fd.sock);
           }
         msgp += sprintf (msgp, "\n");
         _dbus_verbose ("%s",msg);
@@ -1421,13 +1416,13 @@ _dbus_poll (DBusPollFD *fds,
 
             fdp->revents = 0;
 
-            if (FD_ISSET (fdp->fd, &read_set))
+            if (FD_ISSET (fdp->fd.sock, &read_set))
               fdp->revents |= _DBUS_POLLIN;
 
-            if (FD_ISSET (fdp->fd, &write_set))
+            if (FD_ISSET (fdp->fd.sock, &write_set))
               fdp->revents |= _DBUS_POLLOUT;
 
-            if (FD_ISSET (fdp->fd, &err_set))
+            if (FD_ISSET (fdp->fd.sock, &err_set))
               fdp->revents |= _DBUS_POLLERR;
           }
       }
@@ -1491,7 +1486,7 @@ _dbus_exit (int code)
  * @param error return location for error code
  * @returns connection file descriptor or -1 on error
  */
-int
+DBusSocket
 _dbus_connect_tcp_socket (const char     *host,
                           const char     *port,
                           const char     *family,
@@ -1500,14 +1495,15 @@ _dbus_connect_tcp_socket (const char     *host,
   return _dbus_connect_tcp_socket_with_nonce (host, port, family, (const char*)NULL, error);
 }
 
-int
+DBusSocket
 _dbus_connect_tcp_socket_with_nonce (const char     *host,
                                      const char     *port,
                                      const char     *family,
                                      const char     *noncefile,
                                      DBusError      *error)
 {
-  int fd = -1, res;
+  DBusSocket fd = DBUS_SOCKET_INIT;
+  int res;
   struct addrinfo hints;
   struct addrinfo *ai, *tmp;
 
@@ -1516,7 +1512,7 @@ _dbus_connect_tcp_socket_with_nonce (const char     *host,
   if (!_dbus_win_startup_winsock ())
     {
       _DBUS_SET_OOM (error);
-      return -1;
+      return _dbus_socket_get_invalid ();
     }
 
   _DBUS_ZERO (hints);
@@ -1532,7 +1528,7 @@ _dbus_connect_tcp_socket_with_nonce (const char     *host,
       dbus_set_error (error,
                       DBUS_ERROR_INVALID_ARGS,
                       "Unknown address family %s", family);
-      return -1;
+      return _dbus_socket_get_invalid ();
     }
   hints.ai_protocol = IPPROTO_TCP;
   hints.ai_socktype = SOCK_STREAM;
@@ -1548,13 +1544,13 @@ _dbus_connect_tcp_socket_with_nonce (const char     *host,
                       _dbus_error_from_errno (res),
                       "Failed to lookup host/port: \"%s:%s\": %s (%d)",
                       host, port, _dbus_strerror(res), res);
-      return -1;
+      return _dbus_socket_get_invalid ();
     }
 
   tmp = ai;
   while (tmp)
     {
-      if ((fd = socket (tmp->ai_family, SOCK_STREAM, 0)) == INVALID_SOCKET)
+      if ((fd.sock = socket (tmp->ai_family, SOCK_STREAM, 0)) == INVALID_SOCKET)
         {
           DBUS_SOCKET_SET_ERRNO ();
           dbus_set_error (error,
@@ -1562,15 +1558,15 @@ _dbus_connect_tcp_socket_with_nonce (const char     *host,
                           "Failed to open socket: %s",
                           _dbus_strerror_from_errno ());
           freeaddrinfo(ai);
-          return -1;
+          return _dbus_socket_get_invalid ();
         }
       _DBUS_ASSERT_ERROR_IS_CLEAR(error);
 
-      if (connect (fd, (struct sockaddr*) tmp->ai_addr, tmp->ai_addrlen) == SOCKET_ERROR)
+      if (connect (fd.sock, (struct sockaddr*) tmp->ai_addr, tmp->ai_addrlen) == SOCKET_ERROR)
         {
           DBUS_SOCKET_SET_ERRNO ();
-          closesocket(fd);
-          fd = -1;
+          closesocket(fd.sock);
+          fd.sock = INVALID_SOCKET;
           tmp = tmp->ai_next;
           continue;
         }
@@ -1579,13 +1575,13 @@ _dbus_connect_tcp_socket_with_nonce (const char     *host,
     }
   freeaddrinfo(ai);
 
-  if (fd == -1)
+  if (!_dbus_socket_is_valid (fd))
     {
       dbus_set_error (error,
                       _dbus_error_from_errno (errno),
                       "Failed to connect to socket \"%s:%s\" %s",
                       host, port, _dbus_strerror_from_errno ());
-      return -1;
+      return _dbus_socket_get_invalid ();
     }
 
   if (noncefile != NULL)
@@ -1595,9 +1591,9 @@ _dbus_connect_tcp_socket_with_nonce (const char     *host,
       if (!_dbus_string_init (&noncefileStr) ||
           !_dbus_string_append(&noncefileStr, noncefile))
         {
-          closesocket (fd);
+          closesocket (fd.sock);
           dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-          return -1;
+          return _dbus_socket_get_invalid ();
         }
 
       ret = _dbus_send_nonce (fd, &noncefileStr, error);
@@ -1606,18 +1602,18 @@ _dbus_connect_tcp_socket_with_nonce (const char     *host,
 
       if (!ret)
         {
-          closesocket (fd);
-          return -1;
+          closesocket (fd.sock);
+          return _dbus_socket_get_invalid ();
         }
     }
 
   /* Every SOCKET is also a HANDLE. */
-  _dbus_win_handle_set_close_on_exec ((HANDLE) fd);
+  _dbus_win_handle_set_close_on_exec ((HANDLE) fd.sock);
 
-  if (!_dbus_set_fd_nonblocking (fd, error))
+  if (!_dbus_set_socket_nonblocking (fd, error))
     {
-      closesocket (fd);
-      return -1;
+      closesocket (fd.sock);
+      return _dbus_socket_get_invalid ();
     }
 
   return fd;
@@ -1643,10 +1639,11 @@ _dbus_listen_tcp_socket (const char     *host,
                          const char     *port,
                          const char     *family,
                          DBusString     *retport,
-                         int           **fds_p,
+                         DBusSocket    **fds_p,
                          DBusError      *error)
 {
-  int nlisten_fd = 0, *listen_fd = NULL, res, i, port_num = -1;
+  DBusSocket *listen_fd = NULL;
+  int nlisten_fd = 0, res, i, port_num = -1;
   struct addrinfo hints;
   struct addrinfo *ai, *tmp;
 
@@ -1706,8 +1703,8 @@ _dbus_listen_tcp_socket (const char     *host,
   tmp = ai;
   while (tmp)
     {
-      int fd = -1, *newlisten_fd;
-      if ((fd = socket (tmp->ai_family, SOCK_STREAM, 0)) == INVALID_SOCKET)
+      DBusSocket fd = DBUS_SOCKET_INIT, *newlisten_fd;
+      if ((fd.sock = socket (tmp->ai_family, SOCK_STREAM, 0)) == INVALID_SOCKET)
         {
           DBUS_SOCKET_SET_ERRNO ();
           dbus_set_error (error,
@@ -1718,10 +1715,10 @@ _dbus_listen_tcp_socket (const char     *host,
         }
       _DBUS_ASSERT_ERROR_IS_CLEAR(error);
 
-      if (bind (fd, (struct sockaddr*) tmp->ai_addr, tmp->ai_addrlen) == SOCKET_ERROR)
+      if (bind (fd.sock, (struct sockaddr*) tmp->ai_addr, tmp->ai_addrlen) == SOCKET_ERROR)
         {
           DBUS_SOCKET_SET_ERRNO ();
-          closesocket (fd);
+          closesocket (fd.sock);
           if (errno == WSAEADDRINUSE)
           {
               /* Calling this function with port=0 tries to
@@ -1737,20 +1734,20 @@ _dbus_listen_tcp_socket (const char     *host,
           goto failed;
     }
 
-      if (listen (fd, 30 /* backlog */) == SOCKET_ERROR)
+      if (listen (fd.sock, 30 /* backlog */) == SOCKET_ERROR)
         {
           DBUS_SOCKET_SET_ERRNO ();
           dbus_set_error (error, _dbus_error_from_errno (errno),
                           "Failed to listen on socket \"%s:%s\": %s",
                           host ? host : "*", port, _dbus_strerror_from_errno ());
-          closesocket (fd);
+          closesocket (fd.sock);
           goto failed;
         }
 
-      newlisten_fd = dbus_realloc(listen_fd, sizeof(int)*(nlisten_fd+1));
+      newlisten_fd = dbus_realloc(listen_fd, sizeof(DBusSocket)*(nlisten_fd+1));
       if (!newlisten_fd)
         {
-          closesocket (fd);
+          closesocket (fd.sock);
           dbus_set_error (error, DBUS_ERROR_NO_MEMORY,
                           "Failed to allocate file handle array");
           goto failed;
@@ -1771,7 +1768,7 @@ _dbus_listen_tcp_socket (const char     *host,
               socklen_t addrlen = sizeof(addr);
               char portbuf[NI_MAXSERV];
 
-              if (getsockname(fd, &addr.Address, &addrlen) == SOCKET_ERROR ||
+              if (getsockname(fd.sock, &addr.Address, &addrlen) == SOCKET_ERROR ||
                 (res = getnameinfo (&addr.Address, addrlen, NULL, 0,
                                     portbuf, sizeof(portbuf),
                                     NI_NUMERICSERV)) != 0)
@@ -1821,8 +1818,8 @@ _dbus_listen_tcp_socket (const char     *host,
 
   for (i = 0 ; i < nlisten_fd ; i++)
     {
-      _dbus_win_handle_set_close_on_exec ((HANDLE) listen_fd[i]);
-      if (!_dbus_set_fd_nonblocking (listen_fd[i], error))
+      _dbus_win_handle_set_close_on_exec ((HANDLE) listen_fd[i].sock);
+      if (!_dbus_set_socket_nonblocking (listen_fd[i], error))
         {
           goto failed;
         }
@@ -1836,7 +1833,7 @@ _dbus_listen_tcp_socket (const char     *host,
   if (ai)
     freeaddrinfo(ai);
   for (i = 0 ; i < nlisten_fd ; i++)
-    closesocket (listen_fd[i]);
+    closesocket (listen_fd[i].sock);
   dbus_free(listen_fd);
   return -1;
 }
@@ -1849,22 +1846,22 @@ _dbus_listen_tcp_socket (const char     *host,
  * @param listen_fd the listen file descriptor
  * @returns the connection fd of the client, or -1 on error
  */
-int
-_dbus_accept  (int listen_fd)
+DBusSocket
+_dbus_accept  (DBusSocket listen_fd)
 {
-  int client_fd;
+  DBusSocket client_fd;
 
  retry:
-  client_fd = accept (listen_fd, NULL, NULL);
+  client_fd.sock = accept (listen_fd.sock, NULL, NULL);
 
-  if (DBUS_SOCKET_IS_INVALID (client_fd))
+  if (!_dbus_socket_is_valid (client_fd))
     {
       DBUS_SOCKET_SET_ERRNO ();
       if (errno == EINTR)
         goto retry;
     }
 
-  _dbus_verbose ("client fd %d accepted\n", client_fd);
+  _dbus_verbose ("client fd %Iu accepted\n", client_fd.sock);
   
   return client_fd;
 }
@@ -1873,8 +1870,8 @@ _dbus_accept  (int listen_fd)
 
 
 dbus_bool_t
-_dbus_send_credentials_socket (int            handle,
-                        DBusError      *error)
+_dbus_send_credentials_socket (DBusSocket      handle,
+                               DBusError      *error)
 {
 /* FIXME: for the session bus credentials shouldn't matter (?), but
  * for the system bus they are presumably essential. A rough outline
@@ -1950,7 +1947,7 @@ again:
  * @returns #TRUE on success
  */
 dbus_bool_t
-_dbus_read_credentials_socket  (int              handle,
+_dbus_read_credentials_socket  (DBusSocket       handle,
                                 DBusCredentials *credentials,
                                 DBusError       *error)
 {
@@ -1972,7 +1969,7 @@ _dbus_read_credentials_socket  (int              handle,
       _dbus_string_free (&buf);
     }
 
-  pid = _dbus_get_peer_pid_from_tcp_handle (handle);
+  pid = _dbus_get_peer_pid_from_tcp_handle (handle.sock);
   if (pid == 0)
     return TRUE;
 
@@ -2256,11 +2253,13 @@ _dbus_create_directory (const DBusString *filename,
  *
  * @param str the string
  * @param n_bytes the number of random bytes to append to string
- * @returns #TRUE on success, #FALSE if no memory
+ * @param error location to store reason for failure
+ * @returns #TRUE on success
  */
 dbus_bool_t
 _dbus_generate_random_bytes (DBusString *str,
-                             int         n_bytes)
+                             int         n_bytes,
+                             DBusError  *error)
 {
   int old_len;
   char *p;
@@ -2269,15 +2268,22 @@ _dbus_generate_random_bytes (DBusString *str,
   old_len = _dbus_string_get_length (str);
 
   if (!_dbus_string_lengthen (str, n_bytes))
-    return FALSE;
+    {
+      _DBUS_SET_OOM (error);
+      return FALSE;
+    }
 
   p = _dbus_string_get_data_len (str, old_len, n_bytes);
 
   if (!CryptAcquireContext (&hprov, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT))
-    return FALSE;
+    {
+      _DBUS_SET_OOM (error);
+      return FALSE;
+    }
 
   if (!CryptGenRandom (hprov, n_bytes, p))
     {
+      _DBUS_SET_OOM (error);
       CryptReleaseContext (hprov, 0);
       return FALSE;
     }
@@ -2963,8 +2969,11 @@ _dbus_daemon_publish_session_bus_address (const char* address, const char *scope
     }
 
   // create shm
+  dbus_uint64_t len = strlen( address ) + 1;
+
   hDBusSharedMem = CreateFileMappingA( INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,
-                                       0, strlen( address ) + 1, _dbus_string_get_const_data(&shm_name) );
+                                       len >> 32, len & 0xffffffffu,
+                                       _dbus_string_get_const_data(&shm_name) );
   _dbus_assert( hDBusSharedMem );
 
   shared_addr = MapViewOfFile( hDBusSharedMem, FILE_MAP_WRITE, 0, 0, 0 );
