@@ -27,7 +27,7 @@
 #define DBUS_SYSDEPS_WIN_H
 
 extern void *_dbus_win_get_dll_hmodule (void);
-#define _WINSOCKAPI_
+#define WIN32_LEAN_AND_MEAN
 
 #include "dbus-hash.h"
 #include "dbus-string.h"
@@ -40,13 +40,16 @@ extern void *_dbus_win_get_dll_hmodule (void);
 
 
 void _dbus_win_set_errno (int err);
+DBUS_PRIVATE_EXPORT
 const char* _dbus_win_error_from_last_error (void);
 
-void _dbus_win_startup_winsock (void);
+dbus_bool_t _dbus_win_startup_winsock (void);
 void _dbus_win_warn_win_error  (const char *message,
-                                int         code);
+                                unsigned long code);
                                 
+DBUS_PRIVATE_EXPORT
 char * _dbus_win_error_string (int error_number);
+DBUS_PRIVATE_EXPORT
 void _dbus_win_free_error_string (char *string);
 
 extern const char* _dbus_lm_strerror  (int error_number);
@@ -70,6 +73,7 @@ wchar_t    *_dbus_win_utf8_to_utf16 (const char  *str,
 char       *_dbus_win_utf16_to_utf8 (const wchar_t *str,
                                      DBusError *error);
 
+DBUS_PRIVATE_EXPORT
 void        _dbus_win_set_error_from_win_error (DBusError *error, int code);
 
 dbus_bool_t
@@ -80,14 +84,18 @@ _dbus_win_sid_to_name_and_domain (dbus_uid_t uid,
 
 dbus_bool_t _dbus_file_exists (const char *filename);
 
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_get_config_file_name(DBusString *config_file, 
                                        char *s);
 
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_get_install_root(char *prefix, int len);
 
 void        _dbus_threads_windows_init_global (void);
 void        _dbus_threads_windows_ensure_ctor_linked (void);
 
+DBUS_PRIVATE_EXPORT
+dbus_bool_t _dbus_getsid(char **sid, dbus_pid_t process_id);
 #endif
 
 /** @} end of sysdeps-win.h */
